package cashmachine.impl;

import java.util.List;
import java.util.ArrayList;

import cashmachine.api.AccountService;
import cashmachine.api.CashMachineException;
import cashmachine.model.Account;

public class SimpleAccountService implements AccountService
{
	private class SAS_Account
	{
		private Account account;
		private int balance;
		private boolean banned = false;
		
		private SAS_Account wdAccount = null;
		private int wdNumber = 0;
		private int wdBalanceDiff = 0;
		
		public SAS_Account(Account account)
		{
			this.account = account;
			this.balance = account.getBalance();
		}
		
		public Account getAccount()
		{
			return account;
		}
		
		public Account getPublicAccount()
		{
			return new Account(account.getAccountId(), 0, account.getBalance(),
					account.getLimit(), account.getAuthorizedAccountIds());
		}
		
		public boolean setBanned(boolean banned)
		{
			return (this.banned = banned);
		}
		
		public boolean getBanned()
		{
			return banned;
		}
		
		public int getBalance()
		{
			return balance;
		}
		
		public boolean withdraw(SAS_Account account, int transid, int amount)
		{
			if (account == null || transid <= 0 || amount <= 0)
				return false;
			if (account.getAccount().getLimit() < amount || account.getBalance() < amount)
				return false;
			
			wdAccount     = account;
			wdNumber      = transid;
			wdBalanceDiff = amount;
			wdAccount.balance -= amount;
			return true;
		}
		
		public boolean revertWithdraw(int transid)
		{
			if (transid <= 0 || transid != this.wdNumber)
				return false;
			wdAccount.balance += wdBalanceDiff;
			wdBalanceDiff = wdNumber = 0;
			wdAccount = null;
			return true;
		}
	}
	
	/*--------------------------------------------------------------------------*/
	
	private List<SAS_Account> accounts;
	private int transid = 0;
	
	/*--------------------------------------------------------------------------*/
	
	public SimpleAccountService(List<Account> accounts)
	{
		this.accounts = new ArrayList<SAS_Account>();
		for(Account account : accounts)
			this.accounts.add(new SAS_Account(account));
	}
	
	/*--------------------------------------------------------------------------*/
	
	public boolean banAccount(int accountid)
	{
		for(SAS_Account account : accounts)
		{
			if (account.getAccount().getAccountId() != accountid)
				continue;
			return (account.getBanned() || account.setBanned(true));
		}
		return false;
	}
	
	/*--------------------------------------------------------------------------*/
	
	private SAS_Account getWithdrawalSASAccount(int accountid, int pin, int withdrawalAccountid)
	{
		Account cardaccount = validatePin(accountid, pin);
		if (cardaccount == null)
			return null;
		
		if (!cardaccount.getAuthorizedAccountIds().contains(withdrawalAccountid))
			return null;
		
		for(SAS_Account account : accounts)
		{
			if (account.getAccount().getAccountId() != withdrawalAccountid)
				continue;
			return account;
		}
		
		return null;
	}
	/*--------------------------------------------------------------------------*/
	
	public Account getWithdrawalAccount(int accountid, int pin, int withdrawalAccountid)
	{
		SAS_Account cardaccount = getWithdrawalSASAccount(accountid, pin, withdrawalAccountid);
		if (cardaccount == null)
			return null;
		return cardaccount.getPublicAccount();
	}
	
	/*--------------------------------------------------------------------------*/
	
	public boolean isValidAccountId(int accountid)
	{
		for(SAS_Account account : accounts)
		{
			if (account.getAccount().getAccountId() != accountid)
				continue;
			return (!account.getBanned());
		}
		return false;
	}
	
	/*--------------------------------------------------------------------------*/
	
	public boolean revertWithdraw(int accountid, int pin, int transactionNumber)
	throws CashMachineException
	{
		SAS_Account account = validatePinSAS(accountid, pin);
		if (account == null)
			throw new CashMachineException();
		return account.revertWithdraw(transactionNumber);
	}
	
	/*--------------------------------------------------------------------------*/
	
	private SAS_Account validatePinSAS(int accountid, int pin)
	{
		for(SAS_Account account : accounts)
		{
			if (account.getAccount().getAccountId() != accountid)
				continue;
			if (account.getAccount().getPin() != pin)
				return null;
			return account;
		}
		return null;
	}
	/*--------------------------------------------------------------------------*/
	
	public Account validatePin(int accountid, int pin)
	{
		SAS_Account account = validatePinSAS(accountid, pin);
		if (account == null)
			return null;
		return account.getPublicAccount();
	}
	
	/*--------------------------------------------------------------------------*/
	
	public int withdraw(int accountid, int pin, int withdrawalAccountid, int amount)
	throws CashMachineException
	{
		SAS_Account account = validatePinSAS(accountid, pin);
		SAS_Account wdAccount = getWithdrawalSASAccount(accountid, pin, withdrawalAccountid);
		if (account == null || wdAccount == null || amount <= 0)
			throw new CashMachineException();
		
		if (account.withdraw(wdAccount, ++transid, amount))
			return transid;
		return 0;
	}
}
